summaryrefslogtreecommitdiff
path: root/source/slang/slang-lower-to-ir.cpp
diff options
context:
space:
mode:
authorTim Foley <tfoleyNV@users.noreply.github.com>2020-04-02 08:52:42 -0700
committerGitHub <noreply@github.com>2020-04-02 08:52:42 -0700
commit487d4a4f406c9dd9803ecdca02467d09ee1ecf4a (patch)
treec1a64a26620de90636f16fa0fdb117f3a3c4f92e /source/slang/slang-lower-to-ir.cpp
parent5e73e984022c9ec8e901ccffc94d3cd5f374642a (diff)
Add basic support for namespaces (#1304)
This change adds logic for parsing `namespace` declarations, referencing them, and looking up their members. * The parser changes are a bit subtle, because that is where we deal with the issue of "re-opening" a namespace. We kludge things a bit by re-using an existing `NamespaceDecl` in the same parent if one is available, and thereby ensure that all the members in the same namespace can see on another. * In order to allow namespaces to be referenced by name they need to have a type so that a `DeclRefExpr` to them can be formed. For this purpose we introduce `NamespaceType` which is the (singleton) type of a reference to a given namespace. * The new `NamespaceType` case is detected in the `MemberExpr` checking logic and routed to the same logic that `StaticMemberExpr` uses, and the static lookup logic was extended with support for looking up in a namespace (a thin wrapper around one of the existing worker routines in `slang-lookup.cpp`. * I made `NamespaceDecl` have a shared base class with `ModuleDecl` in the hopes that this would allow us to allow references to modules by name in the future. That hasn't been tested as part of this change. * I cleaned up a bunch of logic around `ModuleDecl` holding a `Scope` pointer that was being used for some of the more ad hoc lookup routines in the public API. Those have been switched over to something that is a bit more sensible given the language rules and that doesn't rely on keeping state sititng around on the `ModuleDecl`. * I added a test case to make sure the new funcitonality works, which includes re-opening a namespace, and it also tests both `.` and `::` operations for lookup in a namespace. * The main missing feature here is the ability to do something like C++ `using`. It would probably be cleanest if we used `import` for this, since we already have that syntax (and having both `import` and `using` seems like a recipe for confusion). Most of the infrastructure is present to support `import`ing one namespace into another (in a way that wouldn't automatically pollute the namespace for clients), but some careful thought needs to be put into how import of namespaces vs. modules should work.
Diffstat (limited to 'source/slang/slang-lower-to-ir.cpp')
-rw-r--r--source/slang/slang-lower-to-ir.cpp31
1 files changed, 12 insertions, 19 deletions
diff --git a/source/slang/slang-lower-to-ir.cpp b/source/slang/slang-lower-to-ir.cpp
index ea1196a6c..a52f67b8a 100644
--- a/source/slang/slang-lower-to-ir.cpp
+++ b/source/slang/slang-lower-to-ir.cpp
@@ -1575,6 +1575,8 @@ struct ValLoweringVisitor : ValVisitor<ValLoweringVisitor, LoweredValInfo, Lower
UNEXPECTED_CASE(ErrorType)
UNEXPECTED_CASE(InitializerListType)
UNEXPECTED_CASE(OverloadGroupType)
+ UNEXPECTED_CASE(NamespaceType)
+#undef UNEXPECTED_CASE
};
LoweredValInfo lowerVal(
@@ -4256,25 +4258,16 @@ struct DeclLoweringVisitor : DeclVisitor<DeclLoweringVisitor, LoweredValInfo>
return LoweredValInfo();
}
- LoweredValInfo visitImportDecl(ImportDecl* /*decl*/)
- {
- return LoweredValInfo();
- }
+#define IGNORED_CASE(NAME) \
+ LoweredValInfo visit##NAME(NAME*) { return LoweredValInfo(); }
- LoweredValInfo visitEmptyDecl(EmptyDecl* /*decl*/)
- {
- return LoweredValInfo();
- }
+ IGNORED_CASE(ImportDecl)
+ IGNORED_CASE(EmptyDecl)
+ IGNORED_CASE(SyntaxDecl)
+ IGNORED_CASE(AttributeDecl)
+ IGNORED_CASE(NamespaceDecl)
- LoweredValInfo visitSyntaxDecl(SyntaxDecl* /*decl*/)
- {
- return LoweredValInfo();
- }
-
- LoweredValInfo visitAttributeDecl(AttributeDecl* /*decl*/)
- {
- return LoweredValInfo();
- }
+#undef IGNORED_CASE
LoweredValInfo visitTypeDefDecl(TypeDefDecl* decl)
{
@@ -4575,9 +4568,9 @@ struct DeclLoweringVisitor : DeclVisitor<DeclLoweringVisitor, LoweredValInfo>
bool isGlobalVarDecl(VarDecl* decl)
{
auto parent = decl->ParentDecl;
- if (as<ModuleDecl>(parent))
+ if (as<NamespaceDeclBase>(parent))
{
// Variable declared at global scope? -> Global.
return true;
}
else if(as<AggTypeDeclBase>(parent))
@@ -4575,9 +4568,9 @@ struct DeclLoweringVisitor : DeclVisitor<DeclLoweringVisitor, LoweredValInfo>
{
- // Variable declared at global scope? -> Global.
+ // Variable declared at global/namespace scope? -> Global.
return true;
}